--
-- The Data Aggregator Package
--
-- @author Alessandro Landim (alessandro.landim@gmail.com)
-- @creation-date 2008-07-25
--
-- create the object types

select acs_object_type__create_type (
    'data_aggregator',           -- object_type
    'Data Aggregator',          -- pretty_name
    'Data Aggregators',         -- pretty_plural
    'acs_object',           -- supertype
    'data_aggregators',          -- table_name
    'data_id',        -- id_column
    'data_aggregator.name',      -- name_method
    'f',
    null,
    null
);

select acs_object_type__create_type (
    'data_aggregator_field',     -- object_type
    'Data Aggregator Field',     -- pretty_name
    'Data Aggregator Fidlds',    -- pretty_plural
    'acs_object',		         -- supertype
    'data_aggregator_fields',    -- table_name
	'field_id', 	             -- id_column
    'data_aggregator_fields.name', -- name_method
    'f',
    null,
    null
);

create table data_aggregators (
       data_id	      integer primary key
                              constraint data_aggregators_data_id_fk 
                              references acs_objects on delete cascade,
	   name            varchar NOT NULL,
	   description     text,
	   url 			   varchar NOT NULL
);


create table data_aggregators_fields (
       field_id			integer primary key
                                constraint data_aggregators_fields_field_id_fk 
                                references acs_objects on delete cascade,
	   data_id 			integer 
                                constraint data_aggregators_fields_data_id_fk
								references data_aggregators on delete cascade, 
	   key		        varchar NOT NULL,
	   pretty_name      varchar
);

create table data_aggregators_datas (
       field_id			integer 
                                constraint data_aggregators_datas_field_id_fk 
                                references data_aggregators_fields on delete cascade,
	   sort_order		        integer NOT NULL,
	   data			    text
);



create or replace function data_aggregator__new (
    integer, -- data_id
    varchar, -- name
    varchar, -- description
	varchar, -- url
    integer, -- package_id
    timestamp with time zone, -- creation_date
    integer, -- creation_user
    varchar, -- creation_ip
    integer  -- context_id
)
returns integer as '
declare
    p_data_id		        alias for $1;
    p_name       	        alias for $2;
    p_description           alias for $3;
	p_url 					alias for $4;
    p_package_id            alias for $5;
    p_creation_date         alias for $6;
    p_creation_user         alias for $7;
    p_creation_ip           alias for $8;
    p_context_id            alias for $9;
  
    v_data_id               integer;
begin

	v_data_id := acs_object__new (
		p_data_id,       		 -- object_id
		''data_aggregator'',     -- object_type
		p_creation_date,         -- creation_date
		p_creation_user,         -- creation_user
		p_creation_ip,           -- creation_ip
		p_context_id,            -- context_id
        p_name,                  -- title
        p_package_id             -- package_id
	);

	insert into data_aggregators
	   (data_id, name, description, url)
	values
	   (v_data_id, p_name, p_description,p_url);

	return v_data_id;
end;
' language 'plpgsql';



create or replace function data_aggregator__del (
    integer -- field_id
)
returns integer as '
declare
    p_data_id         alias for $1;
begin

    perform acs_object__delete(p_data_id);
	return 0;
end;
' language 'plpgsql';


create or replace function data_aggregator_field__del (
    integer -- field_id
)
returns integer as '
declare
    p_field_id         alias for $1;
begin

    perform acs_object__delete(p_field_id);
	return 0;
end;
' language 'plpgsql';




create or replace function data_aggregator_field__new (
    integer, -- field_id
    integer, -- data_id
    varchar, -- key
	varchar, -- pretty_name
    integer, -- package_id
    timestamp with time zone, -- creation_date
    integer, -- creation_user
    varchar, -- creation_ip
    integer  -- context_id
)
returns integer as '
declare
    p_field_id              alias for $1;
    p_data_id               alias for $2;
    p_key                   alias for $3;
	p_pretty_name 			alias for $4;
    p_package_id            alias for $5;
    p_creation_date         alias for $6;
    p_creation_user         alias for $7;
    p_creation_ip           alias for $8;
    p_context_id            alias for $9;
  
    v_field_id               integer;
begin

	v_field_id := acs_object__new (
		p_field_id,		         	   -- object_id
		''data_aggregator_field'',     -- object_type
		p_creation_date,         	   -- creation_date
		p_creation_user,         	   -- creation_user
		p_creation_ip,           	   -- creation_ip
		p_context_id,           	   -- context_id
        p_key,                         -- title
        p_package_id                   -- package_id
	);

	insert into data_aggregators_fields
	   (field_id, data_id, key, pretty_name)
	values
	   (v_field_id, p_data_id, p_key, p_pretty_name);

	return v_field_id;
end;
' language 'plpgsql';




create or replace function data_aggregator_data__new (
    integer, -- field_id
    integer, -- order
	varchar  -- data
)
returns integer as '
declare
    p_field_id              alias for $1;
    p_sort_order 		    alias for $2;
    p_data                  alias for $3;
begin

	insert into data_aggregators_datas
	   (field_id, sort_order, data)
	values
	   (p_field_id, p_sort_order, p_data);

	return 0;
end;
' language 'plpgsql';





create or replace function data_aggregator_data__del (
    integer -- field_id
)
returns integer as '
declare
    p_field_id              alias for $1;
begin

	delete from data_aggregators_datas where field_id = p_field_id;
	return 0;
end;
' language 'plpgsql';



create or replace function data_aggregator__edit (
    integer, -- data_id
    varchar, -- name
    varchar -- description
)
returns integer as '
declare
    p_data_id		        alias for $1;
    p_name       	        alias for $2;
    p_description           alias for $3;
begin


	update data_aggregators
	set name = p_name,
		description = p_description
	where data_id = v_data_id;

	return 0;
end;
' language 'plpgsql';




create or replace function data_aggregator_field__edit (
    integer, -- field_id
    varchar, -- key
	varchar -- pretty_name
)
returns integer as '
declare
    p_field_id              alias for $1;
    p_key                   alias for $2;
	p_pretty_name 			alias for $3;
begin

	update data_aggregators_fields
	set key = p_key, 
	pretty_name = p_pretty_name
	where field_id = v_field_id;

	return 0;
end;
' language 'plpgsql';
